@@ -47,8 +47,8 @@ def test_sigalrm(testdir):
             time.sleep(2)
      """
     )
-    result = testdir.runpytest("--timeout=1.5")
-    result.stdout.fnmatch_lines(["*Failed: Timeout >1.5s*"])
+    result = testdir.runpytest("--timeout=1")
+    result.stdout.fnmatch_lines(["*Failed: Timeout >1.0s*"])
 
 
 def test_thread(testdir):
@@ -77,6 +77,7 @@ def test_thread(testdir):
 )
 def test_cov(testdir):
     # This test requires pytest-cov
+    pytest.importorskip("pytest_cov")
     testdir.makepyfile(
         """
         import time
@@ -86,7 +87,7 @@ def test_cov(testdir):
     """
     )
     result = testdir.runpytest(
-        "--timeout=1", "--cov=test_cov.py", "--timeout-method=thread"
+        "--timeout=1", "--cov=test_cov", "--timeout-method=thread"
     )
     result.stderr.fnmatch_lines(
         [
@@ -150,7 +151,7 @@ def test_fix_setup(meth, scope, testdir):
             scope=scope
         )
     )
-    result = testdir.runpytest("--timeout=1", "--timeout-method={}".format(meth))
+    result = testdir.runpytest("--timeout=1", f"--timeout-method={meth}")
     assert result.ret > 0
     assert "Timeout" in result.stdout.str() + result.stderr.str()
 
@@ -164,7 +165,7 @@ def test_fix_setup_func_only(testdir):
 
             @pytest.fixture
             def fix(self):
-                time.sleep(2)
+                time.sleep(0.1)
 
             @pytest.mark.timeout(func_only=True)
             def test_foo(self, fix):
@@ -197,7 +198,7 @@ def test_fix_finalizer(meth, scope, testdir):
                 pass
     """
     )
-    result = testdir.runpytest("--timeout=1", "-s", "--timeout-method={}".format(meth))
+    result = testdir.runpytest("--timeout=1", "-s", f"--timeout-method={meth}")
     assert result.ret > 0
     assert "Timeout" in result.stdout.str() + result.stderr.str()
 
@@ -214,7 +215,7 @@ def test_fix_finalizer_func_only(testdir):
                 print('fix setup')
                 def fin():
                     print('fix finaliser')
-                    time.sleep(2)
+                    time.sleep(0.1)
                 request.addfinalizer(fin)
 
             @pytest.mark.timeout(func_only=True)
@@ -262,7 +263,7 @@ def test_timeout_mark_non_int(testdir):
         """
      import time, pytest
 
-     @pytest.mark.timeout(0.5)
+     @pytest.mark.timeout(0.01)
      def test_foo():
          time.sleep(1)
     """
@@ -339,7 +340,7 @@ def test_ini_timeout(testdir):
     testdir.makeini(
         """
         [pytest]
-        timeout = 1.5
+        timeout = 1
     """
     )
     result = testdir.runpytest()
@@ -354,7 +355,6 @@ def test_ini_timeout_func_only(testdir):
         @pytest.fixture
         def slow():
             time.sleep(2)
-
         def test_foo(slow):
             pass
     """
@@ -362,7 +362,31 @@ def test_ini_timeout_func_only(testdir):
     testdir.makeini(
         """
         [pytest]
-        timeout = 1.5
+        timeout = 1
+        timeout_func_only = true
+    """
+    )
+    result = testdir.runpytest()
+    assert result.ret == 0
+
+
+def test_ini_timeout_func_only_marker_override(testdir):
+    testdir.makepyfile(
+        """
+        import time, pytest
+
+        @pytest.fixture
+        def slow():
+            time.sleep(2)
+        @pytest.mark.timeout(1.5)
+        def test_foo(slow):
+            pass
+    """
+    )
+    testdir.makeini(
+        """
+        [pytest]
+        timeout = 1
         timeout_func_only = true
     """
     )
@@ -402,7 +426,6 @@ def test_timeout_marker_inheritance(testdir):
             def test_foo_2(self):
                 time.sleep(2)
 
-            @pytest.mark.timeout(timeout=3)
             def test_foo_1(self):
                 time.sleep(1)
     """
@@ -453,7 +476,7 @@ def test_suppresses_timeout_when_debugger_is_entered(
     )
     child = testdir.spawn_pytest(str(p1))
     child.expect("test_foo")
-    time.sleep(2)
+    time.sleep(0.2)
     child.send("c\n")
     child.sendeof()
     result = child.read().decode().lower()
@@ -461,3 +484,128 @@ def test_suppresses_timeout_when_debugger_is_entered(
         child.terminate(force=True)
     assert "timeout >1.0s" not in result
     assert "fail" not in result
+
+
+@pytest.mark.parametrize(
+    ["debugging_module", "debugging_set_trace"],
+    [
+        ("pdb", "set_trace()"),
+        pytest.param(
+            "ipdb",
+            "set_trace()",
+            marks=pytest.mark.xfail(
+                reason="waiting on https://github.com/pytest-dev/pytest/pull/7207"
+                " to allow proper testing"
+            ),
+        ),
+        pytest.param(
+            "pydevd",
+            "settrace(port=4678)",
+            marks=pytest.mark.xfail(reason="in need of way to setup pydevd server"),
+        ),
+    ],
+)
+@have_spawn
+def test_disable_debugger_detection_flag(
+    testdir, debugging_module, debugging_set_trace
+):
+    p1 = testdir.makepyfile(
+        """
+        import pytest, {debugging_module}
+
+        @pytest.mark.timeout(1)
+        def test_foo():
+            {debugging_module}.{debugging_set_trace}
+    """.format(
+            debugging_module=debugging_module, debugging_set_trace=debugging_set_trace
+        )
+    )
+    child = testdir.spawn_pytest(f"{p1} --timeout-disable-debugger-detection")
+    child.expect("test_foo")
+    time.sleep(1.2)
+    result = child.read().decode().lower()
+    if child.isalive():
+        child.terminate(force=True)
+    assert "timeout >1.0s" in result
+    assert "fail" in result
+
+
+def test_is_debugging(monkeypatch):
+    import pytest_timeout
+
+    assert not pytest_timeout.is_debugging()
+
+    # create a fake module named "custom.pydevd" with a trace function on it
+    from types import ModuleType
+
+    module_name = "custom.pydevd"
+    module = ModuleType(module_name)
+    monkeypatch.setitem(sys.modules, module_name, module)
+
+    def custom_trace(*args):
+        pass
+
+    custom_trace.__module__ = module_name
+    module.custom_trace = custom_trace
+
+    assert pytest_timeout.is_debugging(custom_trace)
+
+
+def test_not_main_thread(testdir):
+    testdir.makepyfile(
+        """
+        import threading
+        import pytest_timeout
+
+        current_timeout_setup = pytest_timeout.timeout_setup
+
+        def new_timeout_setup(item):
+            threading.Thread(
+                target=current_timeout_setup, args=(item),
+            ).join()
+
+        pytest_timeout.timeout_setup = new_timeout_setup
+
+        def test_x(): pass
+    """
+    )
+    result = testdir.runpytest("--timeout=1")
+    result.stdout.fnmatch_lines(
+        ["timeout: 1.0s", "timeout method:*", "timeout func_only:*"]
+    )
+
+
+def test_plugin_interface(testdir):
+    testdir.makeconftest(
+        """
+     import pytest
+
+     @pytest.mark.tryfirst
+     def pytest_timeout_set_timer(item, settings):
+         print()
+         print("pytest_timeout_set_timer")
+         return True
+
+     @pytest.mark.tryfirst
+     def pytest_timeout_cancel_timer(item):
+         print()
+         print("pytest_timeout_cancel_timer")
+         return True
+    """
+    )
+    testdir.makepyfile(
+        """
+     import pytest
+
+     @pytest.mark.timeout(1)
+     def test_foo():
+         pass
+    """
+    )
+    result = testdir.runpytest("-s")
+    result.stdout.fnmatch_lines(
+        [
+            "pytest_timeout_set_timer",
+            "pytest_timeout_cancel_timer",
+        ]
+    )
